@@ -185,6 +185,7 @@ class DQ(Leaf):
 class Param(Leaf):
     def __init__(self, data):
         self.data = data
+        super(Param, self).__init__()
 
 
 class Func(Leaf):
@@ -328,6 +329,16 @@ class DecimalField(Field):
                 return value
             return decimal.Decimal(str(value))
 
+def format_unicode(s, encoding='utf-8'):
+    if isinstance(s, unicode):
+        return s
+    elif isinstance(s, basestring):
+        return s.decode(encoding)
+    elif hasattr(s, '__unicode__'):
+        return s.__unicode__()
+    else:
+        return unicode(bytes(s), encoding)
+
 class CharField(Field):
     db_field = 'string'
     template = '%(column_type)s(%(max_length)s)'
@@ -336,14 +347,14 @@ class CharField(Field):
         return {'max_length': 255}
 
     def coerce(self, value):
-        value = unicode(value or '')
+        value = format_unicode(value or '')
         return value[:self.attributes['max_length']]
 
 class TextField(Field):
     db_field = 'text'
 
     def coerce(self, value):
-        return unicode(value or '')
+        return format_unicode(value or '')
 
 def format_date_time(value, formats, post_process=None):
     post_process = post_process or (lambda x: x)
@@ -427,7 +438,7 @@ class RelationDescriptor(FieldDescriptor):
 
     def get_object_or_id(self, instance):
         rel_id = instance._data.get(self.att_name)
-        if rel_id or self.att_name in instance._obj_cache:
+        if rel_id is not None or self.att_name in instance._obj_cache:
             if self.att_name not in instance._obj_cache:
                 obj = self.rel_model.get(self.rel_model._meta.primary_key==rel_id)
                 instance._obj_cache[self.att_name] = obj
@@ -487,6 +498,9 @@ class ForeignKeyField(IntegerField):
 
         if self.rel_model == 'self':
             self.rel_model = self.model_class
+        if self.related_name in self.rel_model._meta.fields:
+            raise AttributeError('Foreign key: %s.%s related name "%s" collision with field of same name' % (
+                self.model_class._meta.name, self.name, self.related_name))
 
         setattr(model_class, name, RelationDescriptor(self, self.rel_model))
         setattr(self.rel_model, self.related_name, ReverseRelationDescriptor(self))
@@ -609,7 +623,8 @@ class QueryCompiler(object):
         elif isinstance(expr, SelectQuery):
             max_alias = self._max_alias(alias_map)
             clone = expr.clone()
-            clone._select = (clone.model_class._meta.primary_key,)
+            if not expr._explicit_selection:
+                clone._select = (clone.model_class._meta.primary_key,)
             subselect, p = self.parse_select_query(clone, max_alias, alias_map)
             s = '(%s)' % subselect
         elif isinstance(expr, (list, tuple)):
@@ -697,6 +712,8 @@ class QueryCompiler(object):
 
     def parse_select_query(self, query, start=1, alias_map=None):
         model = query.model_class
+        db = model._meta.database
+
         alias_map = alias_map or {}
         alias_map.update(self.calculate_alias_map(query, start))
 
@@ -737,8 +754,9 @@ class QueryCompiler(object):
             order_by, _ = self.parse_expr_list(query._order_by, alias_map)
             parts.append('ORDER BY %s' % order_by)
 
-        if query._limit:
-            parts.append('LIMIT %s' % query._limit)
+        if query._limit or (query._offset and not db.empty_limit):
+            limit = query._limit or -1
+            parts.append('LIMIT %s' % limit)
         if query._offset:
             parts.append('OFFSET %s' % query._offset)
         if query._for_update:
@@ -939,7 +957,7 @@ class QueryResultWrapper(object):
                 if not fk_field:
                     continue
 
-                if not joined_inst.get_id() and fk_field.name in inst._data:
+                if joined_inst.get_id() is None and fk_field.name in inst._data:
                     rel_inst_id = inst._data[fk_field.name]
                     joined_inst.set_id(rel_inst_id)
 
@@ -983,6 +1001,15 @@ class QueryResultWrapper(object):
         self.__idx += 1
         return instance
 
+    def fill_cache(self, n=None):
+        n = n or float('Inf')
+        self.__idx = self.__ct
+        while not self._populated and (n > self.__ct):
+            try:
+                self.next()
+            except StopIteration:
+                break
+
 def returns_clone(func):
     def inner(self, *args, **kwargs):
         clone = self.clone()
@@ -1149,6 +1176,7 @@ class SelectQuery(Query):
     require_commit = False
 
     def __init__(self, model_class, *selection):
+        self._explicit_selection = len(selection) > 0
         self._select = self._model_shorthand(selection or model_class._meta.get_fields())
         self._group_by = None
         self._having = None
@@ -1163,6 +1191,7 @@ class SelectQuery(Query):
 
     def clone(self):
         query = super(SelectQuery, self).clone()
+        query._explicit_selection = self._explicit_selection
         query._select = list(self._select)
         if self._group_by is not None:
             query._group_by = list(self._group_by)
@@ -1312,6 +1341,25 @@ class SelectQuery(Query):
     def __iter__(self):
         return iter(self.execute())
 
+    def __getitem__(self, value):
+        offset = limit = None
+        if isinstance(value, slice):
+            if value.start:
+                offset = value.start
+            if value.stop:
+                limit = value.stop - (value.start or 0)
+        else:
+            if value < 0:
+                raise ValueError('Negative indexes are not supported, try ordering in reverse')
+            offset = value
+            limit = 1
+        if self._limit != limit or self._offset != offset:
+            self._qr = None
+        self._limit = limit
+        self._offset = offset
+        res = list(self)
+        return limit == 1 and res[0] or res
+
 
 class UpdateQuery(Query):
     def __init__(self, model_class, update=None):
@@ -1369,6 +1417,7 @@ class DeleteQuery(Query):
 class Database(object):
     commit_select = False
     compiler_class = QueryCompiler
+    empty_limit = False
     field_overrides = {}
     for_update = False
     interpolation = '?'
@@ -1378,7 +1427,8 @@ class Database(object):
     sequences = False
     subquery_delete_same_table = True
 
-    def __init__(self, database, threadlocals=False, autocommit=True, **connect_kwargs):
+    def __init__(self, database, threadlocals=False, autocommit=True,
+                 fields=None, ops=None, **connect_kwargs):
         self.init(database, **connect_kwargs)
 
         if threadlocals:
@@ -1389,6 +1439,9 @@ class Database(object):
         self._conn_lock = threading.Lock()
         self.autocommit = autocommit
 
+        self.field_overrides = dict_update(self.field_overrides, fields or {})
+        self.op_overrides = dict_update(self.op_overrides, ops or {})
+
     def init(self, database, **connect_kwargs):
         self.deferred = database is None
         self.database = database
@@ -1425,6 +1478,14 @@ class Database(object):
     def _connect(self, database, **kwargs):
         raise NotImplementedError
 
+    @classmethod
+    def register_fields(cls, fields):
+        cls.field_overrides = dict_update(cls.field_overrides, fields)
+
+    @classmethod
+    def register_ops(cls, ops):
+        cls.op_overrides = dict_update(cls.op_overrides, ops)
+
     def last_insert_id(self, cursor, model):
         if model._meta.auto_increment:
             return cursor.lastrowid
@@ -1552,6 +1613,7 @@ class SqliteDatabase(Database):
 
 class PostgresqlDatabase(Database):
     commit_select = True
+    empty_limit = True
     field_overrides = {
         'bigint': 'BIGINT',
         'bool': 'BOOLEAN',
@@ -1842,10 +1904,6 @@ class Model(object):
         for k, v in kwargs.items():
             setattr(self, k, v)
 
-    @classmethod
-    def raw(cls, sql, *params):
-        return RawQuery(cls, sql, *params)
-
     @classmethod
     def select(cls, *selection):
         query = SelectQuery(cls, *selection)
@@ -1867,16 +1925,16 @@ class Model(object):
     def delete(cls):
         return DeleteQuery(cls)
 
+    @classmethod
+    def raw(cls, sql, *params):
+        return RawQuery(cls, sql, *params)
+
     @classmethod
     def create(cls, **query):
         inst = cls(**query)
         inst.save(force_insert=True)
         return inst
 
-    @classmethod
-    def filter(cls, *dq, **query):
-        return cls.select().filter(*dq, **query)
-
     @classmethod
     def get(cls, *query, **kwargs):
         sq = cls.select().naive()
@@ -1894,6 +1952,10 @@ class Model(object):
         except cls.DoesNotExist:
             return cls.create(**kwargs)
 
+    @classmethod
+    def filter(cls, *dq, **query):
+        return cls.select().filter(*dq, **query)
+
     @classmethod
     def table_exists(cls):
         return cls._meta.db_table in cls._meta.database.get_tables()
@@ -1936,7 +1998,7 @@ class Model(object):
     def save(self, force_insert=False):
         field_dict = dict(self._data)
         pk = self._meta.primary_key
-        if self.get_id() and not force_insert:
+        if self.get_id() is not None and not force_insert:
             field_dict.pop(pk.name)
             update = self.update(
                 **field_dict
@@ -1977,7 +2039,7 @@ class Model(object):
 
     def __eq__(self, other):
         return other.__class__ == self.__class__ and \
-               self.get_id() and \
+               self.get_id() is not None and \
                other.get_id() == self.get_id()
 
     def __ne__(self, other):
@@ -2002,8 +2064,8 @@ def sort_models_topologically(models):
     def dfs(model):
         if model in models and model not in seen:
             seen.add(model)
-            for child_model in model._meta.reverse_rel.values():
-                dfs(child_model)
+            for foreign_key in model._meta.reverse_rel.values():
+                dfs(foreign_key.model_class)
             ordering.append(model)  # parent will follow descendants
     # order models by name and table initially to guarantee a total ordering
     names = lambda m: (m._meta.name, m._meta.db_table)
